<?php
/*
Plugin Name: 410 for Wordpress
Plugin URI: http://rayofsolaris.net/code/410-for-wordpress
Description: Sends HTTP 410 (Gone) responses to requests for pages that no longer exist on your blog.
Version: 0.5
Author: Samir Shah
Author URI: http://rayofsolaris.net/
License: GPL2
*/

if( ! defined( 'ABSPATH' ) )
	exit;

class WP_410 {
	const db_version = 2; 	// DB version, since 0.2
	private $permalinks;
	
	function __construct() {
		$struct = get_option('permalink_structure');
		$this->permalinks = !empty( $struct );
		
		add_action( 'init',			array( $this, 'upgrade_check' )				);
		
		if( is_admin() )
			add_action( 'admin_menu',	array( $this, 'settings_menu' ) 		);
		
		add_action('delete_post',		array( $this, 'note_deleted_post' )	);
		add_action('wp_insert_post', 	array( $this, 'note_inserted_post' )	);
		add_action('template_redirect', array( $this, 'check_for_410' ) 		);
	}
	
	function upgrade_check() {
		$options_version = get_option( 'wp_410_options_version', 0 );
		
		if( $options_version == self::db_version )	// nothing to do
			return;
			
		$old_links = get_option( 'wp_410_links_list', array() );
		$new_links = array();
		
		if( 0 == $options_version ) {
			// upgrade from 0.1 to 0.2, saving links in regex form
			foreach( $old_links as $link )
				$new_links[ rawurldecode( $link ) ] = self::build_regex( rawurldecode( $link ) );	// new links have key = display link and value = regex
			
		}
		elseif( 1 == $options_version ) {
			// regex format changed, add case-insensitive parameter to end and fix urlencoding
			foreach( $old_links as $key => $regex )
				$new_links[ rawurldecode( $key ) ] = rawurldecode( $regex ) . 'i';
		}

		update_option( 'wp_410_links_list', $new_links );
		update_option( 'wp_410_options_version', self::db_version );
	}
	
	function settings_menu() {
		add_submenu_page('plugins.php', '410 for WordPress', '410 for WordPress', 'manage_options', 'wp_410_settings', array( $this, 'settings_page') );
	}
	
	function settings_page() {
		$old_links = get_option( 'wp_410_links_list', array() );
		
		if ( isset( $_POST['submit'] ) ) {
			$failed_to_add = array();
			if( !empty( $_POST['old_links_to_add'] ) ) {
				$added_links = preg_split( '/(\r?\n)+/', $_POST['old_links_to_add'], -1, PREG_SPLIT_NO_EMPTY );
				
				foreach( $added_links as $k => $link ) {
					$parts = parse_url( $link );
					if( !$parts || empty( $parts['scheme'] ) || empty( $parts['host'] ) ) // invalid URL
						$failed_to_add[] = '<code>' . htmlspecialchars( $link ) . '</code>';
					else
						$old_links[$link] = self::build_regex( $link );	// key is display link, value is regex
				}
			}
			
			if( !empty( $_POST['old_links_to_remove'] ) )
				foreach( $_POST['old_links_to_remove'] as $key )
					unset( $old_links[$key] );
			
			ksort( $old_links );
	
			update_option( 'wp_410_links_list', $old_links );
			echo '<div class="updated fade"><p>Options updated.</p></div>';
			
			if( $failed_to_add )
				echo '<div class="error"><p>The following entries could not be recognised as valid URLs, and were not added to the list.<br />- ' . implode( '<br />- ', $failed_to_add ) .'</p></div>';
		}
	?>
	<style>
	.indent {padding-left: 2em}
	.wp_gone_deleted label {text-decoration: line-through}
	</style>
	<div class="wrap">
	<h2>410 for WordPress</h2>
	<p>This plugin will issue a HTTP 410 response to articles that no longer exist on your blog. When an article is deleted, it records this and issues a 410 response when pages unique to that article are requested. This HTTP code informs robots visiting your site that the requested page has been permanently removed, and that they should stop trying to access it.</p>
	<p>Note: this plugin will only return a 410 response if Wordpress has not found something valid to display for the requested URL.</p>
	<form action="" method="post" id="wp-gone-captcha-settings">
	<table class="form-table"><tbody>
	<tr><th>Obsolete URLs</th><td>
	<?php
	if( empty( $old_links ) ) {
		echo '<p>There are currently no obsolete URLs in this list. You can add some manually below.</p>';
	}
	else {
		echo '<p>The following URLs are currently considered obsolete. To remove items from the list, check the corresponding box.</p>';
		echo '<ul id="wp_gone_old_links" class="indent">';
		foreach( array_keys( $old_links ) as $k )
			echo "<li><input type='checkbox' name='old_links_to_remove[]' value='" . htmlspecialchars( $k ) . "' id='old_link_$k' /> <label for='old_link_$k'><code>" . htmlspecialchars( rawurldecode( $k ) ). "</code></label></li>";
		echo '</ul>';
	}
	?>
	<p><small>If you create or update an article whose URL matches one of those above, that URL will automatically be removed from this list.</small></p>
	</td></tr>
	<tr><th>Manually add URLs</th><td>
	<p>You can manually add items to the list by entering them below. Please enter one <strong>fully qualified</strong> URL per line.</p>
	<p>Use <code>*</code> as a wildcard character. So <code>http://www.example.com/*/music/</code> will match all URLs ending in <code>/music/</code>.</p>
	<textarea name="old_links_to_add" rows="8" cols="80"></textarea>
	</td></tr>
	<tr><th>410 reponse message</th><td>
	<p>By default, the plugin issues the following plain-text message as part of the 410 response: <code>Sorry, the page you requested has been permanently removed.</code></p>
	<?php 
		if( locate_template( '410.php' ) ) 
			echo '<p><strong>A template file <code>410.php</code> has been detected in your theme folder. This file will be used to display 410 responses.</strong> To revert back to the default message, remove the file from your template folder.</p>';
		else 
			echo '<p>If you would like to use your own template instead, simply place a file called <code>410.php</code> in your theme folder, containing your template.</p>';
	?>
	</td></tr>
	</tbody></table>
	<p class="submit"><input class="button-primary" type="submit" name="submit" value="Update settings" /></p>
	</form>
	</div>
	<script>
	jQuery(document).ready(function(){
		jQuery('#wp_gone_old_links input').change( function(){
			this.parentNode.className = jQuery(this).is(":checked") ? 'wp_gone_deleted' : '';
		});
	});
	</script>
<?php
	}
	
	function note_deleted_post( $id ) {
		// NOTE: function wp_delete_post calls the delete_post action twice, so we store deleted IDs in an array to avoid repeating
		// but still allow repeated use of the function for batch deletes
		static $noted = array();
		$id = (int) $id;
		
		if( in_array( $id, $noted ) )
			return;
		$noted[] = $id;
		
		$post = get_post( $id );
		
		if( 'revision' == $post->post_type )
			return;
			
		$link = rawurldecode( get_permalink( $id ) );
		if( $this->permalinks )
			$link .= '*';	// catch everything like /page/ and /feed/

		$list = get_option( 'wp_410_links_list', array() );
		
		$list[$link] = self::build_regex( $link );
		
		update_option( 'wp_410_links_list', $list );
	}
	
	function note_inserted_post( $id ) {
		$post = get_post( $id );
		
		if( 'revision' == $post->post_type || 'draft' == $post->post_status )
			return;
		
		// Check our list of URLs against the new/updated post's permalink, and if they match, scratch it from our list
		$links = array();
		
		$links[] = rawurldecode( get_permalink( $id ) );
		$links[] = get_post_comments_feed_link( $id );	// back compat
				
		if( $this->permalinks )
			$links[0] .= '*';
			
		$list = get_option( 'wp_410_links_list', array() );
		
		foreach( $links as $link )	
			unset( $list[$link] );
	
		update_option( 'wp_410_links_list', $list );
	}
	
	function check_for_410() {
		// Don't mess if Wordpress has found something to display
		if( !is_404() )
			return;
			
		$links = get_option( 'wp_410_links_list', array() );
		$req  = ( is_ssl() ? 'https://' : 'http://' ) . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
		$req = rawurldecode( $req );
		
		foreach( $links as $regex ) {
			if( @preg_match( $regex, $req ) ) {
				status_header( 410 );
				do_action( 'wp_410_response' );	// you can use this to customise the response message
				
				if( ! locate_template( '410.php', true ) )
					echo 'Sorry, the page you requested has been permanently removed.';
					
				exit;
			}
		}
	}
	
	static function build_regex( $link ) {
		$parts = preg_split( '/(\*)/', $link, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY );
		foreach( $parts as &$part ) if( '*' != $part )
			$part = preg_quote( $part, '|' );
		$parts = str_replace( '*', '.*', $parts );
		return '|^' . implode( '', $parts ) . '$|i';
	}
}

new WP_410();
